import { Dialog } from '@jupyterlab/apputils';
import { CompletionHandler } from '@jupyterlab/completer';
import { ISettingRegistry } from '@jupyterlab/settingregistry';
import { DataConnector } from '@jupyterlab/statedb';
import { Token } from '@lumino/coreutils';
import { IDisposable } from '@lumino/disposable';
import { ISignal } from '@lumino/signaling';
import { Widget } from '@lumino/widgets';
/**
 * The OmniSciConnectionManager token.
 */
export declare const IOmniSciConnectionManager: Token<IOmniSciConnectionManager>;
/**
 * A type stub for a connection object.
 */
export declare type OmniSciConnection = any;
/**
 * Connection data for the omnisci browser client.
 *
 * #### Notes
 * This interface is intended to be API compatible with the servers.json
 * server specification that is used by OmniSci Immerse. As such,
 * it includes a number of fields that we do not use in this package.
 */
export interface IOmniSciConnectionData {
    /**
     * The name of the database to connect to.
     */
    database?: string;
    /**
     * Whether this connection should be considered the default one.
     */
    master?: boolean;
    /**
     * Username for the database connection.
     */
    username?: string;
    /**
     * Password for the database connection.
     */
    password?: string;
    /**
     * A URL for the OmniSci server.
     *
     * If host, protocol, and port are given,
     * those will take precedence.
     */
    url?: string;
    /**
     * Custom styles used by Immerse.
     * Typed as `any` here as they are unused.
     */
    customStyles?: any;
    /**
     * The protocol to use when connecting.
     */
    protocol?: 'http' | 'https' | string;
    /**
     * Custom styles for mapbox.
     * Unused here.
     */
    mapboxCustomStyles?: any;
    /**
     * The host URL for the connection.
     */
    host?: string;
    /**
     * The port for the connection.
     */
    port?: number | string;
    /**
     * GTM string.
     */
    GTM?: string;
    /**
     * The dashboard to load in Immerse.
     * Not used here.
     */
    loadDashboard?: any;
}
/**
 * The public interface for a connection manager.
 */
export interface IOmniSciConnectionManager extends IDisposable {
    /**
     * The default connection data.
     */
    readonly defaultConnection: IOmniSciConnectionData | undefined;
    /**
     * A connection specifying properties that are stored
     * in environment variables. For places where code is submitted
     * to kernels, this may be used to pull from environment variables
     * instead of supplying credentials from the settings system.
     *
     * #### Notes
     * This is not used for client-side connections like the SQL editor,
     * as they do not have access to environment variables.
     */
    readonly environment: IOmniSciConnectionData | undefined;
    /**
     * A list of predefined connections.
     */
    readonly connections: ReadonlyArray<IOmniSciConnectionData>;
    /**
     * Prompt user to choose a connection.
     */
    chooseConnection(label: string, oldData?: IOmniSciConnectionData): Promise<IOmniSciConnectionData | undefined>;
    /**
     * A signal that fires when the connection listing changes.
     */
    readonly changed: ISignal<this, void>;
}
export declare class OmniSciConnectionManager implements IOmniSciConnectionManager {
    /**
     * Construct a new conection manager.
     */
    constructor(options: OmniSciConnectionManager.IOptions);
    /**
     * The default connection data.
     *
     * #### Notes
     * Setting the default triggers an asynchronous write to the settings
     * system. The changed signal will not fire until that is complete.
     *
     * Setting the default to `undefined` will not necessarily produce
     * the expected behavior! The connection manager will still look through the
     * servers known to it, and select one that is marked with `"master": true`,
     * or, if there is only one option, select that.
     */
    defaultConnection: IOmniSciConnectionData | undefined;
    /**
     * A connection specifying properties that are stored
     * in environment variables. For places where code is submitted
     * to kernels, this may be used to pull from environment variables
     * instead of supplying credentials from the settings system.
     *
     * #### Notes
     * This is not used for client-side connections like the SQL editor,
     * as they do not have access to environment variables.
     */
    environment: IOmniSciConnectionData | undefined;
    /**
     * The overall list of connections known to the manager.
     */
    readonly connections: ReadonlyArray<IOmniSciConnectionData>;
    /**
     * A signal that fires when the list of connections changes.
     */
    readonly changed: ISignal<this, void>;
    /**
     * Prompt user to choose a connection.
     */
    chooseConnection(label: string, oldData?: IOmniSciConnectionData): Promise<IOmniSciConnectionData | undefined>;
    /**
     * Prompt the user to populate environment variables.
     */
    setEnvironment(): Promise<IOmniSciConnectionData | undefined>;
    /**
     * Dispose of the connection manager.
     */
    dispose(): void;
    /**
     * Whether the connection manager is disposed.
     */
    readonly isDisposed: boolean;
    /**
     * React to the settings changing.
     * This emits the `changed` signal once it is done.
     */
    private _onSettingsChanged;
    private _settings;
    private _isDisposed;
    private _defaultConnection;
    private _environment;
    private _changed;
    private _labConnections;
}
/**
 * A namespace for OmniSciConnectionManager statics.
 */
export declare namespace OmniSciConnectionManager {
    /**
     * Options for creating a connection manager.
     */
    interface IOptions {
        /**
         * A settings object which gets connection data from the server.
         */
        settings: ISettingRegistry.ISettings;
    }
}
/**
 * Make a connection to the OmniSci backend.
 *
 * @param data: connection data for the OmniSci database.
 *   Must include at least protocol, host, and port. If a session ID
 *   is not given, it must also incldue database, usernamem and password.
 *
 * @param sessionId: an optional session ID for an already-authenticated
 *   database session.
 *
 * @returns a promise that resolves with the connection object.
 */
export declare function makeConnection(data: IOmniSciConnectionData, sessionId?: string): Promise<OmniSciConnection>;
/**
 * A dialog for entering OmniSci connection data.
 */
export declare class OmniSciConnectionDialog extends Widget implements Dialog.IBodyWidget<IOmniSciConnectionData> {
    constructor(options?: OmniSciConnectionDialog.IOptions);
    /**
     * Get connection data for the current state of the dialog.
     */
    getValue(): IOmniSciConnectionData;
    handleEvent(event: Event): void;
    /**
     * Handle `after-attach` messages for the widget.
     */
    protected onAfterAttach(): void;
    /**
     * Handle `before-detach` messages for the widget.
     */
    protected onBeforeDetach(): void;
    /**
     * On a change to the select element,
     * populate the input elements with the appropriate
     * data.
     */
    private _evtSelectChange;
    /**
     * Given connection data, populate the inputs.
     */
    private _populateInputs;
    /**
     * Given a list of servers, build a select element
     * with those options. The first option in the select
     * is the null "-" option, for when the inputs don't match.
     * The values for the options are their indices in the list-1
     */
    private _buildSelect;
    private _servers;
    private _select;
    private _user;
    private _password;
    private _database;
    private _host;
    private _protocol;
    private _port;
}
/**
 * A namespace for OmniSciConnectionDialog statics.
 */
export declare namespace OmniSciConnectionDialog {
    /**
     * Options to create a new connection dialog.
     */
    interface IOptions {
        /**
         * A previous connection to prepopulate.
         */
        oldData?: IOmniSciConnectionData;
        /**
         * A list of known servers for selection.
         */
        knownServers?: ReadonlyArray<IOmniSciConnectionData>;
        /**
         * Whether to show the password field.
         *
         * Defaults to `false`.
         */
        showPassword?: boolean;
    }
}
/**
 * A class for fetching completion data from a OmniSci connection.
 */
export declare class OmniSciCompletionConnector extends DataConnector<CompletionHandler.IReply, void, CompletionHandler.IRequest> {
    /**
     * Construct a new completion connector.
     */
    constructor(options?: OmniSciCompletionConnector.IOptions);
    /**
     * Fetch completion data from the OmniSci backend.
     */
    fetch(request: CompletionHandler.IRequest): Promise<CompletionHandler.IReply | undefined>;
    private _connection;
}
/**
 * A namespace for OmniSciCompletionConnector statics.
 */
export declare namespace OmniSciCompletionConnector {
    /**
     * Options used to create the completion connector.
     */
    interface IOptions {
        /**
         * Connection data for the backend.
         */
        connection?: IOmniSciConnectionData;
        /**
         * A session ID for an already authenticated session.
         */
        sessionId?: string;
    }
}
