"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const apputils_1 = require("@jupyterlab/apputils");
const coreutils_1 = require("@jupyterlab/coreutils");
const statedb_1 = require("@jupyterlab/statedb");
const coreutils_2 = require("@lumino/coreutils");
const signaling_1 = require("@lumino/signaling");
const widgets_1 = require("@lumino/widgets");
// tslint:disable-next-line:no-var-requires
require('@mapd/connector/dist/browser-connector');
/* tslint:disable */
/**
 * The OmniSciConnectionManager token.
 */
exports.IOmniSciConnectionManager = new coreutils_2.Token('jupyterlab-omnisci:IOmniSciConnectionManager');
class OmniSciConnectionManager {
    /**
     * Construct a new conection manager.
     */
    constructor(options) {
        this._isDisposed = false;
        this._defaultConnection = undefined;
        this._environment = undefined;
        this._changed = new signaling_1.Signal(this);
        this._labConnections = [];
        this._settings = options.settings;
        this._settings.changed.connect(this._onSettingsChanged, this);
        this._onSettingsChanged(this._settings);
    }
    /**
     * The default connection data.
     *
     * #### Notes
     * Setting the default triggers an asynchronous write to the settings
     * system. The changed signal will not fire until that is complete.
     *
     * Setting the default to `undefined` will not necessarily produce
     * the expected behavior! The connection manager will still look through the
     * servers known to it, and select one that is marked with `"master": true`,
     * or, if there is only one option, select that.
     */
    get defaultConnection() {
        return this._defaultConnection;
    }
    set defaultConnection(value) {
        // If the new value is undefined, write the existing values
        // unmodified. This will trigger a possible selection of the
        // value in `_onSettingsChanged.
        if (!value) {
            this._defaultConnection = Private.chooseDefault(this.connections);
            this._changed.emit(void 0);
            return;
        }
        // Do nothing if there is no change.
        if (this._defaultConnection &&
            coreutils_2.JSONExt.deepEqual(this._defaultConnection, value)) {
            return;
        }
        value.master = false; // Temporarily set to false;
        let labServers = this._labConnections.slice();
        // First loop through the existing servers and unset the master attribute.
        labServers.forEach(s => {
            s.master = false;
        });
        // Next loop through the existing servers and see if one already matches
        // the new server.
        const labMatch = labServers.find(s => {
            return (Object.keys(value).filter((key) => value[key] !== s[key]).length === 0);
        });
        // If we found one, set it to the master server.
        if (labMatch) {
            labMatch.master = true;
        }
        else {
            value.master = true;
            labServers = [value, ...labServers];
        }
        void this._settings.set('servers', labServers);
    }
    /**
     * A connection specifying properties that are stored
     * in environment variables. For places where code is submitted
     * to kernels, this may be used to pull from environment variables
     * instead of supplying credentials from the settings system.
     *
     * #### Notes
     * This is not used for client-side connections like the SQL editor,
     * as they do not have access to environment variables.
     */
    get environment() {
        return this._environment;
    }
    set environment(value) {
        if ((!value && !this._environment) ||
            coreutils_2.JSONExt.deepEqual(value, this._environment)) {
            return;
        }
        else if (!value) {
            void this._settings.remove('environment');
            return;
        }
        void this._settings.set('environment', value);
    }
    /**
     * The overall list of connections known to the manager.
     */
    get connections() {
        return this._labConnections.slice();
    }
    /**
     * A signal that fires when the list of connections changes.
     */
    get changed() {
        return this._changed;
    }
    /**
     * Prompt user to choose a connection.
     */
    chooseConnection(label, oldData) {
        return Private.showConnectionDialog({
            title: label,
            oldData,
            knownServers: this.connections,
            showPassword: false
        });
    }
    /**
     * Prompt the user to populate environment variables.
     */
    setEnvironment() {
        return Private.showConnectionDialog({
            title: 'Set Connection Environment Variables',
            oldData: this._environment,
            showPassword: true
        });
    }
    /**
     * Dispose of the connection manager.
     */
    dispose() {
        this._isDisposed = true;
        signaling_1.Signal.clearData(this);
    }
    /**
     * Whether the connection manager is disposed.
     */
    get isDisposed() {
        return this._isDisposed;
    }
    /**
     * React to the settings changing.
     * This emits the `changed` signal once it is done.
     */
    _onSettingsChanged(settings) {
        const newServers = settings.get('servers').composite || [];
        // Normalize the connection data.
        this._labConnections = newServers.map(Private.normalizeConnectionData);
        const environment = settings.get('environment').composite;
        this._environment = environment;
        this._defaultConnection = Private.chooseDefault(this.connections);
        this._changed.emit(void 0);
    }
}
exports.OmniSciConnectionManager = OmniSciConnectionManager;
/**
 * Make a connection to the OmniSci backend.
 *
 * @param data: connection data for the OmniSci database.
 *   Must include at least protocol, host, and port. If a session ID
 *   is not given, it must also incldue database, usernamem and password.
 *
 * @param sessionId: an optional session ID for an already-authenticated
 *   database session.
 *
 * @returns a promise that resolves with the connection object.
 */
function makeConnection(data, sessionId) {
    return __awaiter(this, void 0, void 0, function* () {
        // Whether or not we have a session id, we need protocol,
        // host, and port to be defined.
        let con = new MapdCon()
            .protocol(data.protocol)
            .host(data.host)
            .port(data.port);
        if (sessionId) {
            // Set fake dbname and user arrays if we are provided a session ID.
            // These are not necessary to make the connection, but the initClients
            // function checks for them anyways.
            // Once the clients have been initialized, set the session id,
            // but do *not* call connect, as that requires username, db and password.
            con = con.dbName(['']).user(['']);
            return con.initClients().sessionId([sessionId]);
        }
        else {
            // If we don't have a session id, provide user authentication.
            con = con
                .dbName(data.database)
                .user(data.username)
                .password(data.password);
        }
        return yield con.connectAsync();
    });
}
exports.makeConnection = makeConnection;
/**
 * A dialog for entering OmniSci connection data.
 */
class OmniSciConnectionDialog extends widgets_1.Widget {
    constructor(options = {}) {
        super();
        let layout = (this.layout = new widgets_1.PanelLayout());
        const oldData = options.oldData;
        this._servers = options.knownServers || [];
        const showPassword = !!options.showPassword;
        if (this._servers.length) {
            this._select = this._buildSelect(this._servers);
            const knownServersLabel = new widgets_1.Widget();
            knownServersLabel.node.textContent = 'Known Servers';
            layout.addWidget(knownServersLabel);
            layout.addWidget(new widgets_1.Widget({ node: this._select }));
        }
        this._user = document.createElement('input');
        this._password = document.createElement('input');
        this._database = document.createElement('input');
        this._host = document.createElement('input');
        this._protocol = document.createElement('input');
        this._port = document.createElement('input');
        this._user.placeholder = 'User name';
        this._password.placeholder = 'Password';
        if (!showPassword) {
            this._password.setAttribute('type', 'password');
        }
        this._database.placeholder = 'Database name';
        this._host.placeholder = 'Host name';
        this._protocol.placeholder = 'Protocol';
        this._port.placeholder = 'Port';
        if (oldData) {
            this._populateInputs(oldData);
        }
        const userLabel = new widgets_1.Widget();
        userLabel.node.textContent = 'User';
        const passwordLabel = new widgets_1.Widget();
        passwordLabel.node.textContent = 'Password';
        const databaseLabel = new widgets_1.Widget();
        databaseLabel.node.textContent = 'Database';
        const hostLabel = new widgets_1.Widget();
        hostLabel.node.textContent = 'Host';
        const protocolLabel = new widgets_1.Widget();
        protocolLabel.node.textContent = 'Protocol';
        const portLabel = new widgets_1.Widget();
        portLabel.node.textContent = 'Port';
        layout.addWidget(userLabel);
        layout.addWidget(new widgets_1.Widget({ node: this._user }));
        layout.addWidget(passwordLabel);
        layout.addWidget(new widgets_1.Widget({ node: this._password }));
        layout.addWidget(databaseLabel);
        layout.addWidget(new widgets_1.Widget({ node: this._database }));
        layout.addWidget(hostLabel);
        layout.addWidget(new widgets_1.Widget({ node: this._host }));
        layout.addWidget(protocolLabel);
        layout.addWidget(new widgets_1.Widget({ node: this._protocol }));
        layout.addWidget(portLabel);
        layout.addWidget(new widgets_1.Widget({ node: this._port }));
    }
    /**
     * Get connection data for the current state of the dialog.
     */
    getValue() {
        const data = {
            username: this._user.value || undefined,
            password: this._password.value || undefined,
            database: this._database.value || undefined,
            host: this._host.value || undefined,
            protocol: this._protocol.value || undefined,
            port: this._port.value ? parseInt(this._port.value, 10) : undefined
        };
        Object.keys(data).forEach((k) => data[k] === undefined && delete data[k]);
        return data;
    }
    handleEvent(event) {
        switch (event.type) {
            case 'change':
                this._evtSelectChange(event);
                break;
            default:
                break;
        }
    }
    /**
     * Handle `after-attach` messages for the widget.
     */
    onAfterAttach() {
        if (this._select) {
            this._select.addEventListener('change', this);
        }
    }
    /**
     * Handle `before-detach` messages for the widget.
     */
    onBeforeDetach() {
        if (this._select) {
            this._select.removeEventListener('change', this);
        }
    }
    /**
     * On a change to the select element,
     * populate the input elements with the appropriate
     * data.
     */
    _evtSelectChange(event) {
        const idx = parseInt(event.target.value, 10);
        if (idx > 0 && idx <= this._servers.length) {
            this._populateInputs(this._servers[idx - 1]);
        }
    }
    /**
     * Given connection data, populate the inputs.
     */
    _populateInputs(data) {
        this._user.value = data.username || '';
        this._password.value = data.password || '';
        this._database.value = data.database || '';
        this._host.value = data.host || '';
        this._protocol.value = data.protocol || '';
        this._port.value = data.port ? `${data.port}` : '';
    }
    /**
     * Given a list of servers, build a select element
     * with those options. The first option in the select
     * is the null "-" option, for when the inputs don't match.
     * The values for the options are their indices in the list-1
     */
    _buildSelect(knownServers) {
        const select = document.createElement('select');
        let idx = 0;
        // Create an option corresponding to none of the known servers.
        const option = document.createElement('option');
        option.value = `${idx++}`;
        option.textContent = '-';
        select.appendChild(option);
        knownServers.forEach(server => {
            const option = document.createElement('option');
            option.value = `${idx++}`;
            option.textContent = server.host || 'Unknown host';
            option.title = `Hostname: ${server.host || 'unknown'}
Protocol: ${server.protocol || 'unknown'}
Port: ${server.port || 'unknown'}
Database: ${server.database || 'unknown'}
User: ${server.username || 'unknown'}
Password ${server.password ? '*****' : 'unknown'}`;
            select.appendChild(option);
        });
        return select;
    }
}
exports.OmniSciConnectionDialog = OmniSciConnectionDialog;
/**
 * A class for fetching completion data from a OmniSci connection.
 */
class OmniSciCompletionConnector extends statedb_1.DataConnector {
    /**
     * Construct a new completion connector.
     */
    constructor(options = {}) {
        super();
        this._connection = undefined;
        // Note: unlike other places, this expects an authenticated
        // session ID to work.
        // TODO: remove this restriction.
        if (options.connection && options.sessionId) {
            this._connection = makeConnection(options.connection, options.sessionId);
        }
    }
    /**
     * Fetch completion data from the OmniSci backend.
     */
    fetch(request) {
        if (!this._connection) {
            return Promise.resolve(void 0);
        }
        return new Promise((resolve, reject) => {
            if (!this._connection) {
                resolve(void 0);
                return;
            }
            this._connection
                .then(con => {
                con.getCompletionHints(request.text, { cursor: request.offset }, (err, result) => {
                    if (err) {
                        throw err;
                    }
                    else if (result && result[0] && result[0].hints) {
                        const matches = result
                            .map((hintObject) => hintObject.hints)
                            .reduce((acc, val) => [].concat(acc, val), []);
                        resolve({
                            start: request.offset - result[0].replaced.length,
                            end: request.offset,
                            matches,
                            metadata: {}
                        });
                        resolve(void 0);
                    }
                    else {
                        resolve(void 0);
                    }
                });
            })
                .catch(err => {
                console.warn('There was an error making a connection to the backend');
                console.warn(err);
                return void 0;
            });
        });
    }
}
exports.OmniSciCompletionConnector = OmniSciCompletionConnector;
/**
 * A namespace for private functionality.
 */
var Private;
(function (Private) {
    /**
     * Given a list of connections, select one as default.
     * This looks for a the first connection indicated with "master: true".
     * If none is found returns the first in the list.
     * If the list is empty, returns undefined.
     */
    function chooseDefault(connections) {
        if (!connections.length) {
            return undefined;
        }
        return connections.find(c => c.master === true) || connections[0];
    }
    Private.chooseDefault = chooseDefault;
    /**
     * Given a valid connection data, normalize it to the format
     * expected in this plugin. In particular, if given a URL to
     * a database, parse it into protocol, host, and port.
     */
    function normalizeConnectionData(data) {
        let { host, port, protocol } = data;
        // Assume https if protocol is undefined.
        protocol = (protocol || 'https').replace(':', '');
        port = port
            ? parseInt(`${port}`, 10)
            : protocol === 'http'
                ? 80
                : protocol === 'https'
                    ? 443
                    : NaN;
        if (data.url) {
            const parsed = coreutils_1.URLExt.parse(data.url);
            protocol = parsed.protocol;
            host = parsed.hostname;
            protocol = (protocol || 'https').replace(':', '');
            // Fill in the port with defaults if necessary.
            if (parsed.port) {
                port = parseInt(parsed.port, 10);
            }
            else {
                port = protocol === 'http' ? 80 : protocol === 'https' ? 443 : NaN;
            }
        }
        return Object.assign({}, data, { host,
            port,
            protocol });
    }
    Private.normalizeConnectionData = normalizeConnectionData;
    /**
     * Show a dialog for entering OmniSci connection data.
     */
    function showConnectionDialog(options) {
        return apputils_1.showDialog({
            title: options.title,
            body: new OmniSciConnectionDialog(options),
            buttons: [apputils_1.Dialog.cancelButton(), apputils_1.Dialog.okButton()]
        }).then(result => {
            if (result.button.accept) {
                return result.value || options.oldData;
            }
            else {
                return options.oldData;
            }
        });
    }
    Private.showConnectionDialog = showConnectionDialog;
})(Private || (Private = {}));
