"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const application_1 = require("@jupyterlab/application");
const apputils_1 = require("@jupyterlab/apputils");
const codeeditor_1 = require("@jupyterlab/codeeditor");
const completer_1 = require("@jupyterlab/completer");
const coreutils_1 = require("@jupyterlab/coreutils");
const launcher_1 = require("@jupyterlab/launcher");
const mainmenu_1 = require("@jupyterlab/mainmenu");
const notebook_1 = require("@jupyterlab/notebook");
const services_1 = require("@jupyterlab/services");
const settingregistry_1 = require("@jupyterlab/settingregistry");
const datagrid_1 = require("@lumino/datagrid");
const connection_1 = require("./connection");
const grid_1 = require("./grid");
const mimeextensions_1 = require("./mimeextensions");
const viewer_1 = require("./viewer");
/**
 * The name of the factory that creates pdf widgets.
 */
const FACTORY = 'OmniSciVega';
/**
 * Command IDs for the extension.
 */
var CommandIDs;
(function (CommandIDs) {
    CommandIDs.newGrid = 'omnisci:new-grid';
    CommandIDs.invokeCompleter = 'omnisci:invoke-completer';
    CommandIDs.selectCompleter = 'omnisci:select-completer';
    CommandIDs.setConnection = 'omnisci:set-connection';
    CommandIDs.setEnvironment = 'omnisci:set-environment';
    CommandIDs.injectIbisConnection = 'omnisci:inject-ibis-connection';
    CommandIDs.createNotebook = 'omnisci:create-notebook';
    CommandIDs.createWorkspace = 'omnisci:create-workspace';
})(CommandIDs || (CommandIDs = {}));
/**
 * The MIME type for Vega.
 *
 * #### Notes
 * The version of this follows the major version of Vega.
 */
exports.VEGA_MIME_TYPE = 'application/vnd.vega.v3+json';
exports.EXTENSIONS = [
    '.vega',
    '.omnisci.vega',
    '.omnisci.vg.json',
    '.omnisci.vega.json',
    '.vg.json',
    '.vega.json'
];
const CONNECTION_PLUGIN_ID = 'jupyterlab-omnisci:connection';
const VEGA_PLUGIN_ID = 'jupyterlab-omnisci:vega';
const SQL_EDITOR_PLUGIN_ID = 'jupyterlab-omnisci:sql-editor';
const NOTEBOOK_PLUGIN_ID = 'jupyterlab-omnisci:notebook';
/**
 * The Omnisci connection handler extension.
 */
const omnisciConnectionPlugin = {
    activate: activateOmniSciConnection,
    id: CONNECTION_PLUGIN_ID,
    requires: [apputils_1.ICommandPalette, mainmenu_1.IMainMenu, settingregistry_1.ISettingRegistry],
    provides: connection_1.IOmniSciConnectionManager,
    autoStart: true
};
function activateOmniSciConnection(app, palette, mainMenu, settingRegistry) {
    return __awaiter(this, void 0, void 0, function* () {
        // Fetch the initial state of the settings.
        const settings = yield settingRegistry.load(CONNECTION_PLUGIN_ID);
        const manager = new connection_1.OmniSciConnectionManager({ settings });
        // Add an application-wide connection-setting command.
        app.commands.addCommand(CommandIDs.setConnection, {
            execute: () => __awaiter(this, void 0, void 0, function* () {
                const connection = yield manager.chooseConnection('Set Default Omnisci Connection', manager.defaultConnection);
                manager.defaultConnection = connection;
                return connection;
            }),
            label: 'Set Default Omnisci Connection...'
        });
        // Add an application-wide connection-setting command.
        app.commands.addCommand(CommandIDs.setEnvironment, {
            execute: () => __awaiter(this, void 0, void 0, function* () {
                const environment = yield manager.setEnvironment();
                manager.environment = environment;
                return environment;
            }),
            label: 'Set OmniSci Connection Environment...'
        });
        mainMenu.settingsMenu.addGroup([
            { command: CommandIDs.setConnection },
            { command: CommandIDs.setEnvironment }
        ], 50);
        palette.addItem({ command: CommandIDs.setConnection, category: 'OmniSci' });
        palette.addItem({ command: CommandIDs.setEnvironment, category: 'OmniSci' });
        return manager;
    });
}
/**
 * The OmniSci-Vega file type.
 */
const omnisciFileType = {
    name: 'omnisci-vega',
    displayName: 'OmniSci Vega',
    fileFormat: 'text',
    extensions: exports.EXTENSIONS,
    mimeTypes: [exports.VEGA_MIME_TYPE],
    iconClass: 'jp-MaterialIcon jp-VegaIcon'
};
/**
 * The Omnisci vega file handler extension.
 */
const omnisciVegaPlugin = {
    activate: activateOmniSciVegaViewer,
    id: VEGA_PLUGIN_ID,
    requires: [application_1.ILayoutRestorer, connection_1.IOmniSciConnectionManager],
    autoStart: true
};
function activateOmniSciVegaViewer(app, restorer, manager) {
    const viewerNamespace = 'omnisci-viewer-widget';
    const factory = new viewer_1.OmniSciVegaViewerFactory({
        name: FACTORY,
        modelName: 'text',
        fileTypes: ['json', 'omnisci-vega', 'vega3', 'vega4'],
        defaultFor: ['omnisci-vega'],
        readOnly: true,
        manager
    });
    const viewerTracker = new apputils_1.WidgetTracker({
        namespace: viewerNamespace
    });
    // Handle state restoration.
    void restorer.restore(viewerTracker, {
        command: 'docmanager:open',
        args: widget => ({ path: widget.context.path, factory: FACTORY }),
        name: widget => widget.context.path
    });
    app.docRegistry.addFileType(omnisciFileType);
    app.docRegistry.addWidgetFactory(factory);
    factory.widgetCreated.connect((sender, widget) => {
        void viewerTracker.add(widget);
        const types = app.docRegistry.getFileTypesForPath(widget.context.path);
        if (types.length > 0) {
            widget.title.iconClass = types[0].iconClass || '';
            widget.title.iconLabel = types[0].iconLabel || '';
        }
    });
    // Update the default connection data for viewers that don't already
    // have it defined.
    manager.changed.connect(() => {
        const defaultConnectionData = manager.defaultConnection;
        viewerTracker.forEach(viewer => {
            if (!viewer.connectionData) {
                viewer.connectionData = defaultConnectionData;
            }
        });
    });
}
/**
 * The Omnisci SQL editor extension.
 */
const omnisciGridPlugin = {
    activate: activateOmniSciGridViewer,
    id: SQL_EDITOR_PLUGIN_ID,
    requires: [
        completer_1.ICompletionManager,
        codeeditor_1.IEditorServices,
        launcher_1.ILauncher,
        application_1.ILayoutRestorer,
        mainmenu_1.IMainMenu,
        connection_1.IOmniSciConnectionManager,
        apputils_1.IThemeManager
    ],
    autoStart: true
};
function activateOmniSciGridViewer(app, completionManager, editorServices, launcher, restorer, mainMenu, manager, themeManager) {
    const gridNamespace = 'omnisci-grid-widget';
    const mimeGridNamespace = 'omnisci-mime-grid-widget';
    const gridTracker = new apputils_1.WidgetTracker({
        namespace: gridNamespace
    });
    // Handle state restoration.
    void restorer.restore(gridTracker, {
        command: CommandIDs.newGrid,
        args: widget => {
            const con = widget.content.grid.connectionData || {};
            const connection = {
                host: con.host || '',
                protocol: con.protocol || '',
                port: con.port || ''
            };
            const sessionId = widget.content.grid.sessionId;
            return {
                initialQuery: widget.content.grid.query,
                connectionData: connection,
                sessionId: sessionId || null
            };
        },
        name: widget => widget.id
    });
    // Create a completion handler for each grid that is created.
    gridTracker.widgetAdded.connect((sender, explorer) => {
        const editor = explorer.content.input.editor;
        const sessionId = explorer.content.grid.sessionId;
        const connector = new connection_1.OmniSciCompletionConnector({
            connection: explorer.content.grid.connectionData,
            sessionId
        });
        const parent = explorer;
        const handle = completionManager.register({ connector, editor, parent });
        explorer.content.grid.onModelChanged.connect(() => {
            const sessionId = explorer.content.grid.sessionId;
            handle.connector = new connection_1.OmniSciCompletionConnector({
                connection: explorer.content.grid.connectionData,
                sessionId
            });
        });
        // Set the theme for the new widget.
        explorer.content.grid.style = style;
        explorer.content.grid.renderer = renderer;
    });
    // The current styles for the data grids.
    let style = Private.LIGHT_STYLE;
    let renderer = Private.LIGHT_RENDERER;
    // Keep the themes up-to-date.
    const updateThemes = () => {
        const isLight = themeManager.theme
            ? themeManager.isLight(themeManager.theme)
            : true;
        style = isLight ? Private.LIGHT_STYLE : Private.DARK_STYLE;
        renderer = isLight ? Private.LIGHT_RENDERER : Private.DARK_RENDERER;
        gridTracker.forEach(grid => {
            grid.content.grid.style = style;
            grid.content.grid.renderer = renderer;
        });
        mimeGridTracker.forEach((mimeGrid) => {
            mimeGrid.widget.content.style = style;
            mimeGrid.widget.content.renderer = renderer;
        });
    };
    themeManager.themeChanged.connect(updateThemes);
    // This is a workaround for some of the limitations of mimerenderer extensions.
    // We want to hook up the theming information and tab-completions to the SQL
    // editor mime renderer, but that requires some full-extension machinery.
    // So we extend the renderer factory with a "created" signal, and when that
    // fires, do some extra work in the real extension.
    const mimeGridTracker = new apputils_1.WidgetTracker({
        namespace: mimeGridNamespace
    });
    // Add the new renderer to an instance tracker when it is created.
    // This will track whether that instance has focus or not.
    mimeextensions_1.sqlEditorRendererFactory.rendererCreated.connect((sender, mime) => {
        void mimeGridTracker.add(mime);
    });
    // When a new grid widget is added, hook up the machinery for
    // completions and theming.
    mimeGridTracker.widgetAdded.connect((sender, mime) => {
        const grid = mime.widget;
        const sessionId = grid.grid.sessionId;
        const editor = grid.input.editor;
        const connector = new connection_1.OmniSciCompletionConnector({
            connection: grid.grid.connectionData,
            sessionId
        });
        const parent = mime;
        const handle = completionManager.register({ connector, editor, parent });
        grid.grid.onModelChanged.connect(() => {
            const sessionId = grid.grid.sessionId;
            handle.connector = new connection_1.OmniSciCompletionConnector({
                connection: grid.grid.connectionData,
                sessionId
            });
        });
        mime.widget.grid.style = style;
        mime.widget.grid.renderer = renderer;
    });
    // Add grid completer command.
    app.commands.addCommand(CommandIDs.invokeCompleter, {
        execute: () => {
            let anchor;
            const current = app.shell.currentWidget;
            if (current && current === gridTracker.currentWidget) {
                anchor = gridTracker.currentWidget;
            }
            else if (current &&
                mimeGridTracker.currentWidget &&
                current.contains(mimeGridTracker.currentWidget)) {
                anchor = mimeGridTracker.currentWidget;
            }
            if (anchor) {
                return app.commands.execute('completer:invoke', { id: anchor.id });
            }
        }
    });
    // Add grid completer select command.
    app.commands.addCommand(CommandIDs.selectCompleter, {
        execute: () => {
            let anchor;
            const current = app.shell.currentWidget;
            if (current && current === gridTracker.currentWidget) {
                anchor = gridTracker.currentWidget;
            }
            else if (current &&
                mimeGridTracker.currentWidget &&
                current.contains(mimeGridTracker.currentWidget)) {
                anchor = mimeGridTracker.currentWidget;
            }
            if (anchor) {
                return app.commands.execute('completer:select', { id: anchor.id });
            }
        }
    });
    // Set enter key for grid completer select command.
    app.commands.addKeyBinding({
        command: CommandIDs.selectCompleter,
        keys: ['Enter'],
        selector: `.omnisci-OmniSci-toolbar .jp-Editor.jp-mod-completer-active`
    });
    app.commands.addKeyBinding({
        command: CommandIDs.invokeCompleter,
        keys: ['Tab'],
        selector: `.omnisci-OmniSci-toolbar .jp-Editor.jp-mod-completer-enabled`
    });
    app.commands.addCommand(CommandIDs.newGrid, {
        label: 'OmniSci SQL Editor',
        iconClass: 'omnisci-OmniSci-logo',
        execute: args => {
            const query = args['initialQuery'] || '';
            const connectionData = args['connectionData'] || undefined;
            const sessionId = args['sessionId'] || undefined;
            const grid = new grid_1.OmniSciSQLEditor({
                editorFactory: editorServices.factoryService.newInlineEditor,
                manager,
                connectionData,
                sessionId,
                initialQuery: query
            });
            Private.id++;
            grid.id = `omnisci-grid-widget-${Private.id}`;
            grid.title.label = `OmniSci SQL Editor ${Private.id}`;
            grid.title.closable = true;
            grid.title.iconClass = 'omnisci-OmniSci-logo';
            const main = new apputils_1.MainAreaWidget({ content: grid });
            main.id = grid.id;
            void gridTracker.add(main);
            app.shell.add(main, 'main');
            app.shell.activateById(main.id);
            grid.grid.onModelChanged.connect(() => {
                void gridTracker.save(main);
            });
            return main;
        }
    });
    mainMenu.fileMenu.newMenu.addGroup([{ command: CommandIDs.newGrid }], 50);
    launcher.add({
        category: 'Other',
        rank: 0,
        command: CommandIDs.newGrid
    });
    // Update the default connection data for grids that don't already
    // have it defined.
    manager.changed.connect(() => {
        const defaultConnectionData = manager.defaultConnection;
        gridTracker.forEach(grid => {
            if (!grid.content.grid.connectionData) {
                void grid.content.grid.setConnectionData(defaultConnectionData);
            }
        });
    });
}
/**
 * The Omnisci inital notebook extension.
 */
const omnisciNotebookPlugin = {
    activate: activateOmniSciNotebook,
    id: NOTEBOOK_PLUGIN_ID,
    requires: [
        application_1.JupyterFrontEnd.IPaths,
        apputils_1.ICommandPalette,
        mainmenu_1.IMainMenu,
        notebook_1.INotebookTracker,
        connection_1.IOmniSciConnectionManager,
        application_1.IRouter
    ],
    autoStart: true
};
function activateOmniSciNotebook(app, paths, palette, menu, tracker, manager, router) {
    // Add a command to inject the ibis connection data into the active notebook.
    app.commands.addCommand(CommandIDs.injectIbisConnection, {
        label: 'Insert Ibis OmniSci Connection…',
        execute: () => __awaiter(this, void 0, void 0, function* () {
            const current = tracker.currentWidget;
            if (!current) {
                return;
            }
            const connection = yield manager.chooseConnection('Choose Ibis Connection', manager.defaultConnection);
            Private.injectIbisConnection({
                notebook: current.content,
                connection,
                environment: manager.environment
            });
        }),
        isEnabled: () => !!tracker.currentWidget
    });
    // Add a command to create a new notebook with an ibis connection.
    app.commands.addCommand(CommandIDs.createNotebook, {
        label: 'Notebook with OmniSci Connection',
        iconClass: 'omnisci-OmniSci-logo',
        execute: (args) => __awaiter(this, void 0, void 0, function* () {
            const connectionData = args['connectionData'] || {};
            const environment = args['environment'] || {};
            const sessionId = args['sessionId'] || '';
            const initialQuery = args['initialQuery'] || '';
            // Create the notebook.
            const notebook = yield app.commands.execute('notebook:create-new', {
                kernelName: 'python3'
            });
            // Move the notebook so it is in a split pane with the primary tab.
            // It has already been added, so this just has the effect of moving it.
            app.shell.add(notebook, 'main');
            yield notebook.context.ready;
            // Define a function for injecting code into the notebook
            // on content changed. This is a somewhat ugly hack, as
            // the notebook model is not entirely ready when the context
            // is ready. Instead, it waits for a new stack frame to add
            // the initial cell. So as a workaround, we wait until there
            // is exactly one cell, then inject our code, then disconnect.
            const inject = () => {
                if (notebook.content.model.cells.length === 1) {
                    notebook.content.model.contentChanged.disconnect(inject);
                    Private.injectIbisConnection({
                        notebook: notebook.content,
                        connection: connectionData,
                        environment,
                        sessionId,
                        initialQuery
                    });
                }
            };
            notebook.content.model.contentChanged.connect(inject);
            return notebook;
        })
    });
    // Add a command to set up a new workspace with a notebook and SQL editor.
    // This specifically does not ask for user input, as we want it to
    // be triggerable via routing.
    app.commands.addCommand(CommandIDs.createWorkspace, {
        label: 'Create OmniSci Workspace',
        execute: () => __awaiter(this, void 0, void 0, function* () {
            yield app.restored;
            const workspace = yield Private.fetchWorkspaceData();
            const connectionData = workspace.connection;
            const environment = workspace.environment;
            const sessionId = workspace.session;
            const initialQuery = workspace.query;
            // Create the SQL editor
            const grid = yield app.commands.execute(CommandIDs.newGrid, {
                initialQuery,
                connectionData,
                sessionId
            });
            // Prefer the environment variables for the notebook creation.
            const notebook = yield app.commands.execute(CommandIDs.createNotebook, {
                environment,
                connectionData: Private.canUseSession(environment)
                    ? {}
                    : connectionData,
                sessionId,
                initialQuery
            });
            // Move the notebook to be side by side with the grid.
            app.shell.add(notebook, 'main', { ref: grid.id, mode: 'split-left' });
        })
    });
    // Add ibis connection injection to the palette and the edit menu.
    palette.addItem({
        command: CommandIDs.injectIbisConnection,
        category: 'OmniSci'
    });
    menu.editMenu.addGroup([{ command: CommandIDs.injectIbisConnection }], 50);
    // Add new notebook creation to the palette and file menu.
    menu.fileMenu.newMenu.addGroup([{ command: CommandIDs.createNotebook }], 11);
    palette.addItem({
        command: CommandIDs.createWorkspace,
        category: 'OmniSci'
    });
    // Add workspace creation to the router, so that external services
    // may launch a new workspace via URL.
    router.register({
        pattern: /(\?omnisci|\&omnisci)($|&)/,
        command: CommandIDs.createWorkspace
    });
}
/**
 * Export the plugin as default.
 */
const plugins = [
    omnisciConnectionPlugin,
    omnisciVegaPlugin,
    omnisciGridPlugin,
    omnisciNotebookPlugin
];
exports.default = plugins;
/**
 * A namespace for private data.
 */
var Private;
(function (Private) {
    /**
     * A counter for widget ids.
     */
    Private.id = 0;
    /**
     * A utility function that checks whether data is enough
     * to connect via session id.
     */
    function canUseSession(data) {
        return !!data && !!data.host && !!data.port && !!data.protocol;
    }
    Private.canUseSession = canUseSession;
    /**
     * Settings for a connection to the server.
     */
    const serverSettings = services_1.ServerConnection.makeSettings();
    function fetchWorkspaceData() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = coreutils_1.URLExt.join(serverSettings.baseUrl, 'omnisci', 'session');
            const response = yield services_1.ServerConnection.makeRequest(url, {}, serverSettings);
            const data = yield response.json();
            return data;
        });
    }
    Private.fetchWorkspaceData = fetchWorkspaceData;
    /**
     * The light theme for the data grid.
     */
    Private.LIGHT_STYLE = Object.assign({}, datagrid_1.DataGrid.defaultStyle, { voidColor: '#F3F3F3', backgroundColor: 'white', headerBackgroundColor: '#EEEEEE', gridLineColor: 'rgba(20, 20, 20, 0.15)', headerGridLineColor: 'rgba(20, 20, 20, 0.25)', rowBackgroundColor: i => (i % 2 === 0 ? '#F5F5F5' : 'white'), selectionFillColor: 'rgba(49, 119, 229, 0.2)', selectionBorderColor: 'rgba(0, 107, 247, 1.0)', cursorBorderColor: 'rgba(0, 107, 247, 1.0)', headerSelectionFillColor: 'rgba(20, 20, 20, 0.1)', scrollShadow: {
            size: 10,
            color1: 'rgba(0, 0, 0, 0.20)',
            color2: 'rgba(0, 0, 0, 0.05)',
            color3: 'rgba(0, 0, 0, 0.00)'
        } });
    /**
     * The dark theme for the data grid.
     */
    Private.DARK_STYLE = {
        voidColor: 'black',
        backgroundColor: '#111111',
        headerBackgroundColor: '#424242',
        gridLineColor: 'rgba(235, 235, 235, 0.15)',
        headerGridLineColor: 'rgba(235, 235, 235, 0.25)',
        rowBackgroundColor: i => (i % 2 === 0 ? '#212121' : '#111111'),
        selectionFillColor: 'rgba(49, 119, 229, 0.2)',
        selectionBorderColor: 'rgba(0, 107, 247, 1.0)',
        cursorBorderColor: 'rgba(0, 107, 247, 1.0)',
        headerSelectionFillColor: 'rgba(20, 20, 20, 0.2)',
        scrollShadow: {
            size: 10,
            color1: 'rgba(0, 0, 0, 0.20)',
            color2: 'rgba(0, 0, 0, 0.05)',
            color3: 'rgba(0, 0, 0, 0.00)'
        }
    };
    /**
     * The light renderer for the data grid.
     */
    Private.LIGHT_RENDERER = new datagrid_1.TextRenderer({
        textColor: '#111111',
        horizontalAlignment: 'right'
    });
    /**
     * The dark renderer for the data grid.
     */
    Private.DARK_RENDERER = new datagrid_1.TextRenderer({
        textColor: '#F5F5F5',
        horizontalAlignment: 'right'
    });
    /**
     * A template for an Ibis OmniSci client.
     */
    const IBIS_TEMPLATE = `
{{os}}import ibis

con = ibis.omniscidb.connect(
    host={{host}}, user={{user}}, password={{password}},
    port={{port}}, database={{database}}, protocol={{protocol}}
)`.trim();
    /**
     * A template for an Ibis OmniSci client when a session ID is available.
     */
    const SESSION_IBIS_TEMPLATE = `
{{os}}import ibis

con = ibis.omniscidb.connect(
    host={{host}}, port={{port}}, protocol={{protocol}}, session_id={{session}}
)`.trim();
    /**
     * String to list tables.
     */
    const LIST_TABLES = '\n\ncon.list_tables()';
    /**
     * Template for an initial query.
     */
    const INITIAL_QUERY = '\n\nexpr = con.sql("{{query}}")';
    /**
     * Construct an ibis connection code snippet and insert it
     * into a notebook cell.
     */
    function injectIbisConnection(options) {
        const notebook = options.notebook;
        const env = options.environment || {};
        const con = {};
        let os = Object.keys(env).length === 0 ? '' : 'import os\n';
        // Merge the connection with any environment variables
        // that have been specified.
        const keys = [
            'host',
            'protocol',
            'port',
            'username',
            'password',
            'database'
        ];
        keys.forEach(key => {
            if (options.connection && options.connection[key]) {
                con[key] = `"${options.connection[key]}"`;
            }
            else if (env[key]) {
                con[key] = `os.environ['${env[key]}']`;
            }
        });
        let value = '';
        if (options.sessionId) {
            value = SESSION_IBIS_TEMPLATE;
            value = value.replace('{{os}}', os);
            value = value.replace('{{host}}', con.host || '""');
            value = value.replace('{{protocol}}', con.protocol || '""');
            value = value.replace('{{session}}', `"${options.sessionId}"`);
            value = value.replace('{{port}}', `${con.port || '""'}`);
        }
        else {
            value = IBIS_TEMPLATE;
            value = value.replace('{{os}}', os);
            value = value.replace('{{host}}', con.host || '""');
            value = value.replace('{{protocol}}', con.protocol || '""');
            value = value.replace('{{password}}', con.password || '""');
            value = value.replace('{{database}}', con.database || '""');
            value = value.replace('{{user}}', con.username || '""');
            value = value.replace('{{port}}', `${con.port || '""'}`);
        }
        // Handle an initial query if given. If not, list tables.
        if (options.initialQuery) {
            value = value + INITIAL_QUERY.replace('{{query}}', options.initialQuery);
        }
        else {
            value = value + LIST_TABLES;
        }
        notebook_1.NotebookActions.insertAbove(options.notebook);
        const model = (notebook.activeCell && notebook.activeCell.model) ||
            notebook.model.cells.get(0);
        // Assert exists because we have verified by creating.
        model.value.text = value;
    }
    Private.injectIbisConnection = injectIbisConnection;
})(Private || (Private = {}));
