import { DataGrid, DataModel, TextRenderer } from '@lumino/datagrid';
import { Message } from '@lumino/messaging';
import { Panel } from '@lumino/widgets';
import { ISignal } from '@lumino/signaling';
import { CodeEditor, CodeEditorWrapper } from '@jupyterlab/codeeditor';
import { IOmniSciConnectionData, IOmniSciConnectionManager } from './connection';
export declare class OmniSciSQLEditor extends Panel {
    /**
     * Construct a new OmniSciSQLEditor widget.
     */
    constructor(options: OmniSciSQLEditor.IOptions);
    /**
     * Get a reference to the input editor.
     */
    readonly input: CodeEditorWrapper;
    /**
     * Get a reference to the grid widget.
     */
    readonly grid: OmniSciGrid;
    /**
     * Handle the DOM events for the widget.
     *
     * @param event - The DOM event sent to the widget.
     *
     * #### Notes
     * This method implements the DOM `EventListener` interface and is
     * called in response to events on the main area widget's node. It should
     * not be called directly by user code.
     */
    handleEvent(event: KeyboardEvent): void;
    /**
     * Handle `after-attach` messages for the widget.
     */
    protected onAfterAttach(msg: Message): void;
    /**
     * Handle `before-detach` messages for the widget.
     */
    protected onBeforeDetach(msg: Message): void;
    /**
     * Handle `'activate-request'` messages.
     */
    protected onActivateRequest(msg: Message): void;
    /**
     * Focus the toolbar editor widget.
     */
    private _focusInput;
    private _content;
    private _grid;
    private _tool;
}
/**
 * A namespace for OmniSciSQLEditor statics.
 */
export declare namespace OmniSciSQLEditor {
    /**
     * Options for creating a new OmniSciSQLEditor.
     */
    interface IOptions {
        /**
         * An editor factory for the SQL editor widget.
         */
        editorFactory: CodeEditor.Factory;
        /**
         * An optional initial connection data structure.
         */
        connectionData?: IOmniSciConnectionData;
        /**
         * An optional pre-authenticated session ID for the SQL editor.
         */
        sessionId?: string;
        /**
         * An optional initial query for the editor.
         */
        initialQuery?: string;
        /**
         * An optional connection manager.
         */
        manager?: IOmniSciConnectionManager;
    }
}
/**
 * A widget that hosts a lumino grid with a OmniSci dataset.
 */
export declare class OmniSciGrid extends Panel {
    /**
     * Construct a new OmniSciGrid widget.
     */
    constructor(options?: OmniSciGrid.IOptions);
    /**
     * The current connection data for the viewer.
     */
    readonly connectionData: IOmniSciConnectionData | undefined;
    setConnectionData(value: IOmniSciConnectionData | undefined, sessionId?: string): Promise<void>;
    /**
     * The current style used by the grid viewer.
     */
    style: DataGrid.Style;
    /**
     * The text renderer for the viewer.
     */
    renderer: TextRenderer;
    /**
     * The query for the viewer.
     */
    query: string;
    /**
     * Get the session ID for the current connection.
     */
    readonly sessionId: string | undefined;
    /**
     * A change signal emitted when the connection or
     * query data change.
     */
    readonly onModelChanged: ISignal<OmniSciGrid, void>;
    /**
     * Update the underlying data model with a new query and connection.
     *
     * If the update fails, either due to a connection failure or a query
     * validation failure, it shows the error in the panel.
     */
    private _updateModel;
    private _model;
    private _grid;
    private _content;
    private _error;
    private _onModelChanged;
}
/**
 * A namespace for OmniSciGrid statics.
 */
export declare namespace OmniSciGrid {
    /**
     * Options for creating a new OmniSciGrid.
     */
    interface IOptions {
        /**
         * An optional initial connection data structure.
         */
        connectionData?: IOmniSciConnectionData;
        /**
         * An optional pre-authenticated session ID for the grid.
         */
        sessionId?: string;
        /**
         * An optional initial query for the editor.
         */
        initialQuery?: string;
    }
}
/**
 * A data model for a query.
 */
export declare class OmniSciTableModel extends DataModel {
    /**
     * Construct a new data model.
     */
    constructor();
    /**
     * Get the number of rows for the model.
     */
    rowCount(region: DataModel.RowRegion): number;
    /**
     * Get the number of columns for the model.
     */
    columnCount(region: DataModel.ColumnRegion): number;
    /**
     * The current connection data for the model.
     */
    readonly connectionData: IOmniSciConnectionData | undefined;
    /**
     * The current query for the viewer.
     */
    readonly query: string;
    /**
     * Get the session ID for the current connection.
     */
    readonly sessionId: string | undefined;
    /**
     * Get data from the model.
     */
    data(region: DataModel.CellRegion, row: number, column: number): any;
    /**
     * Update the model with new connection data or a new query.
     *
     * @param connection - the connection data to use.
     *
     * @param query - the query to use.
     *
     * @returns a promsise that resolves when the model has been updated,
     *   and the connection and query data have been validated. It throws
     *   an error if the validation fails.
     */
    updateModel(connectionData: IOmniSciConnectionData | undefined, query: string, sessionId?: string): Promise<void>;
    /**
     * Reset the model. Should be called when either
     * the query or the connection data change.
     */
    private _updateModel;
    /**
     * Make a query to the database.
     */
    private _makeQuery;
    /**
     * Fetch a block with a given index into memory.
     */
    private _fetchBlock;
    /**
     * Free references to a block when it is no longer needed.
     */
    private _freeBlock;
    /**
     * If we are not chunking the data, then just load the whole thing,
     * limited by DEFAULT_LIMIT.
     */
    private _fetchDataset;
    private _query;
    private _connectionData;
    private _connection;
    private _fieldNames;
    private _dataBlocks;
    private _dataset;
    private _pending;
    private _tableLength;
    private _maxBlock;
    private _streaming;
}
