"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const coreutils_1 = require("@lumino/coreutils");
const datagrid_1 = require("@lumino/datagrid");
const widgets_1 = require("@lumino/widgets");
const signaling_1 = require("@lumino/signaling");
const apputils_1 = require("@jupyterlab/apputils");
const codeeditor_1 = require("@jupyterlab/codeeditor");
const connection_1 = require("./connection");
/**
 * The default block size whenn streaming blocks of the query results.
 */
const BLOCK_SIZE = 50000;
/**
 * The default limit for standalone requests.
 */
const DEFAULT_LIMIT = 50000;
class OmniSciSQLEditor extends widgets_1.Panel {
    /**
     * Construct a new OmniSciSQLEditor widget.
     */
    constructor(options) {
        super();
        const content = new widgets_1.SplitPanel({ orientation: 'vertical', spacing: 2 });
        this.addWidget(content);
        const connection = options.connectionData ||
            (options.manager && options.manager.defaultConnection);
        const grid = new OmniSciGrid({
            connectionData: connection,
            sessionId: options.sessionId,
            initialQuery: options.initialQuery || ''
        });
        const toolbar = Private.createToolbar(grid, options.editorFactory, options.manager);
        content.addWidget(toolbar);
        content.addWidget(grid);
        this._content = content;
        this._grid = grid;
        this._tool = toolbar;
        this.addClass('omnisci-OmniSciSQLEditor');
    }
    /**
     * Get a reference to the input editor.
     */
    get input() {
        return this._tool.children().next();
    }
    /**
     * Get a reference to the grid widget.
     */
    get grid() {
        return this._grid;
    }
    /**
     * Handle the DOM events for the widget.
     *
     * @param event - The DOM event sent to the widget.
     *
     * #### Notes
     * This method implements the DOM `EventListener` interface and is
     * called in response to events on the main area widget's node. It should
     * not be called directly by user code.
     */
    handleEvent(event) {
        switch (event.type) {
            case 'keydown':
                switch (event.keyCode) {
                    case 13: // Enter
                        if (event.shiftKey) {
                            event.stopPropagation();
                            event.preventDefault();
                            this._grid.query = this.input.editor.model.value.text;
                        }
                        break;
                    default:
                        break;
                }
                break;
            default:
                break;
        }
    }
    /**
     * Handle `after-attach` messages for the widget.
     */
    onAfterAttach(msg) {
        this.input.node.addEventListener('keydown', this, true);
        requestAnimationFrame(() => {
            this._content.setRelativeSizes([1, 6]);
        });
        super.onAfterAttach(msg);
    }
    /**
     * Handle `before-detach` messages for the widget.
     */
    onBeforeDetach(msg) {
        this.input.node.removeEventListener('keydown', this, true);
        super.onBeforeDetach(msg);
    }
    /**
     * Handle `'activate-request'` messages.
     */
    onActivateRequest(msg) {
        this._focusInput();
        super.onActivateRequest(msg);
    }
    /**
     * Focus the toolbar editor widget.
     */
    _focusInput() {
        this.input.activate();
    }
}
exports.OmniSciSQLEditor = OmniSciSQLEditor;
/**
 * A widget that hosts a lumino grid with a OmniSci dataset.
 */
class OmniSciGrid extends widgets_1.Panel {
    /**
     * Construct a new OmniSciGrid widget.
     */
    constructor(options = {}) {
        super();
        this._onModelChanged = new signaling_1.Signal(this);
        this.addClass('omnisci-OmniSciGrid');
        // Create the Layout
        this._content = new widgets_1.StackedPanel();
        this._content.addClass('omnisci-OmniSciGrid-content');
        this._error = new widgets_1.Widget({ node: document.createElement('pre') });
        this._error.addClass('omnisci-ErrorMessage');
        this.addWidget(this._content);
        this.addWidget(this._error);
        // Create the data model
        this._model = new OmniSciTableModel();
        // Create the grid
        const renderer = new datagrid_1.TextRenderer({
            textColor: '#111111',
            horizontalAlignment: 'right'
        });
        const gridStyle = Object.assign({}, datagrid_1.DataGrid.defaultStyle, { rowBackgroundColor: i => (i % 2 === 0 ? 'rgba(34, 167, 240, 0.2)' : '') });
        this._grid = new datagrid_1.DataGrid({
            style: gridStyle,
            defaultRenderer: renderer,
            defaultSizes: {
                rowHeight: 24,
                columnWidth: 144,
                rowHeaderWidth: 64,
                columnHeaderHeight: 36
            }
        });
        this._grid.cellRenderers.update(undefined, renderer);
        this._grid.dataModel = this._model;
        this._grid.mouseHandler = new datagrid_1.BasicMouseHandler();
        this._grid.keyHandler = new datagrid_1.BasicKeyHandler();
        this._grid.selectionModel = new datagrid_1.BasicSelectionModel({
            dataModel: this._model
        });
        this._content.addWidget(this._grid);
        this._content.hide(); // Initially hide the grid until we set the query.
        // Initialize the data model.
        void this._updateModel(options.connectionData, options.initialQuery || '', options.sessionId);
    }
    /**
     * The current connection data for the viewer.
     */
    get connectionData() {
        return this._model.connectionData;
    }
    setConnectionData(value, sessionId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._updateModel(value, this._model.query, sessionId);
        });
    }
    /**
     * The current style used by the grid viewer.
     */
    get style() {
        return this._grid.style;
    }
    set style(value) {
        this._grid.style = value;
    }
    /**
     * The text renderer for the viewer.
     */
    get renderer() {
        // Pass in dummy cell position to get back fallback renderer
        return this._grid.cellRenderers.get({});
    }
    set renderer(value) {
        this._grid.cellRenderers.update(undefined, value);
    }
    /**
     * The query for the viewer.
     */
    get query() {
        return this._model.query;
    }
    set query(value) {
        void this._updateModel(this._model.connectionData, value, this._model.sessionId);
    }
    /**
     * Get the session ID for the current connection.
     */
    get sessionId() {
        return this._model.sessionId;
    }
    /**
     * A change signal emitted when the connection or
     * query data change.
     */
    get onModelChanged() {
        return this._onModelChanged;
    }
    /**
     * Update the underlying data model with a new query and connection.
     *
     * If the update fails, either due to a connection failure or a query
     * validation failure, it shows the error in the panel.
     */
    _updateModel(connectionData, query, sessionId) {
        return __awaiter(this, void 0, void 0, function* () {
            const hasQuery = query !== '';
            yield this._model
                .updateModel(connectionData, query, sessionId)
                .then(() => {
                this._content.setHidden(!hasQuery);
                this._error.hide();
                this._error.node.textContent = '';
            })
                .catch((err) => {
                let msg = err.error_msg || err.message || String(err);
                this._content.hide();
                this._error.show();
                this._error.node.textContent = msg;
            });
            this._onModelChanged.emit(void 0);
        });
    }
}
exports.OmniSciGrid = OmniSciGrid;
/**
 * A data model for a query.
 */
class OmniSciTableModel extends datagrid_1.DataModel {
    /**
     * Construct a new data model.
     */
    constructor() {
        super();
        this._query = '';
        this._dataBlocks = {};
        this._dataset = null;
        this._pending = new Set();
        this._tableLength = -1;
        this._maxBlock = Infinity;
        this._streaming = false;
        void this._updateModel();
    }
    /**
     * Get the number of rows for the model.
     */
    rowCount(region) {
        if (region === 'column-header') {
            return 1;
        }
        // If we have found the length of the table, return that.
        if (this._tableLength > 0) {
            return this._tableLength;
        }
        // If we don't know the length, try to infer it from the currently
        // loaded blocks.
        const indices = Object.keys(this._dataBlocks).map(key => Number(key));
        if (indices.length === 0) {
            return 0;
        }
        const maxIndex = Math.max(...indices);
        return BLOCK_SIZE * maxIndex + this._dataBlocks[maxIndex].length;
    }
    /**
     * Get the number of columns for the model.
     */
    columnCount(region) {
        return region === 'body' ? this._fieldNames.length : 1;
    }
    /**
     * The current connection data for the model.
     */
    get connectionData() {
        return this._connectionData;
    }
    /**
     * The current query for the viewer.
     */
    get query() {
        return this._query;
    }
    /**
     * Get the session ID for the current connection.
     */
    get sessionId() {
        if (!this._connection) {
            return undefined;
        }
        const ids = this._connection.sessionId();
        return ids.length ? ids[0] : undefined;
    }
    /**
     * Get data from the model.
     */
    data(region, row, column) {
        if (region === 'row-header') {
            return String(row + 1);
        }
        if (region === 'column-header') {
            return this._fieldNames[column];
        }
        if (region === 'corner-header') {
            return null;
        }
        // If we are streaming data, first check to see if the
        // relevant block is loaded into memory. If it is not,
        // load it. Also load the blocks on each side of the relevant block,
        // and free blocks outside of that window.
        if (this._streaming) {
            const blockIndex = Math.floor(row / BLOCK_SIZE);
            const offset = BLOCK_SIZE * blockIndex;
            const localRow = row - offset;
            // Trash other blocks that we don't need.
            const keep = [0, blockIndex + 1, blockIndex, blockIndex - 1];
            Object.keys(this._dataBlocks).forEach(index => {
                let idx = Number(index);
                if (keep.indexOf(idx) === -1) {
                    this._freeBlock(idx);
                }
            });
            // Check if we should fetch the next block.
            if (blockIndex <= this._maxBlock &&
                localRow / BLOCK_SIZE > 0.9 &&
                !this._dataBlocks[blockIndex + 1]) {
                void this._fetchBlock(blockIndex + 1);
            }
            // Check if we should fetch the previous block.
            if (blockIndex >= 1 &&
                localRow / BLOCK_SIZE < 0.1 &&
                !this._dataBlocks[blockIndex - 1]) {
                void this._fetchBlock(blockIndex - 1);
            }
            // If the current block has not been loaded, then load it and
            // return null. The grid will be notified when it is loaded.
            // If the current block has been loaded, then return the data from it.
            if (!this._dataBlocks[blockIndex]) {
                void this._fetchBlock(blockIndex);
                return null;
            }
            else {
                const block = this._dataBlocks[blockIndex];
                const rowData = block[row - offset];
                return rowData[this._fieldNames[column]];
            }
        }
        else {
            // If we are not streaming, then just return the loaded data.
            const rowData = this._dataset[row];
            return rowData[this._fieldNames[column]];
        }
    }
    /**
     * Update the model with new connection data or a new query.
     *
     * @param connection - the connection data to use.
     *
     * @param query - the query to use.
     *
     * @returns a promsise that resolves when the model has been updated,
     *   and the connection and query data have been validated. It throws
     *   an error if the validation fails.
     */
    updateModel(connectionData, query, sessionId) {
        return __awaiter(this, void 0, void 0, function* () {
            const sameConnection = connectionData &&
                this._connectionData &&
                this._connection &&
                coreutils_1.JSONExt.deepEqual(connectionData, this._connectionData);
            // If nothing has changed, do nothing.
            if (sameConnection && this._query === query) {
                return Promise.resolve(void 0);
            }
            if (!sameConnection) {
                this._connectionData = connectionData;
                this._connection = connectionData
                    ? yield connection_1.makeConnection(connectionData, sessionId)
                    : undefined;
            }
            this._query = query;
            yield this._updateModel();
        });
    }
    /**
     * Reset the model. Should be called when either
     * the query or the connection data change.
     */
    _updateModel() {
        return __awaiter(this, void 0, void 0, function* () {
            // Clear the data of any previous model
            for (let key of Object.keys(this._dataBlocks)) {
                delete this._dataBlocks[Number(key)];
            }
            this._dataset = null;
            this._fieldNames = [];
            this._tableLength = -1;
            this._maxBlock = Infinity;
            this._pending.clear();
            this._streaming = false;
            if (this.query && this.connectionData) {
                this._streaming = Private.shouldChunkRequests(this._query);
                yield this._makeQuery();
            }
            else {
                this.emitChanged({ type: 'model-reset' });
            }
        });
    }
    /**
     * Make a query to the database.
     */
    _makeQuery() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._connection || !this._query) {
                return;
            }
            try {
                // void (await this._connection.validateQuery(this._query));
                this.emitChanged({ type: 'model-reset' });
                if (this._streaming) {
                    return this._fetchBlock(0);
                }
                else {
                    return this._fetchDataset();
                }
            }
            catch (err) {
                this.emitChanged({ type: 'model-reset' });
                throw err;
            }
        });
    }
    /**
     * Fetch a block with a given index into memory.
     */
    _fetchBlock(index) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._connection) {
                return Promise.resolve(void 0);
            }
            // If we are already fetching this block, do nothing.
            if (this._pending.has(index)) {
                return Promise.resolve(void 0);
            }
            this._pending.add(index);
            // Augment the query with the relevant LIMIT and OFFSET.
            const limit = BLOCK_SIZE;
            const offset = index * BLOCK_SIZE;
            const query = `${this._query} LIMIT ${limit} OFFSET ${offset}`;
            const indices = Object.keys(this._dataBlocks).map(key => Number(key));
            const maxIndex = Math.max(...indices);
            const res = yield Private.makeQuery(this._connection, query);
            this._pending.delete(index);
            if (!this._fieldNames.length) {
                this._fieldNames = res.fields.map((field) => field.name);
                this.emitChanged({ type: 'model-reset' });
            }
            this._dataBlocks[index] = res.results;
            if (index <= maxIndex || this._tableLength !== -1) {
                // In this case, we are not appending, so emit a changed
                // signal.
                this.emitChanged({
                    type: 'cells-changed',
                    region: 'body',
                    row: offset,
                    column: 0,
                    rowSpan: res.results.length,
                    columnSpan: this._fieldNames.length
                });
            }
            else {
                if (res.results.length < BLOCK_SIZE) {
                    // If the length of the result is less than the block size,
                    // we have found the table length. Set that.
                    this._tableLength = offset + res.results.length;
                    this._maxBlock = index;
                }
                // Emit a rows-inserted signal.
                this.emitChanged({
                    type: 'rows-inserted',
                    region: 'body',
                    index: offset,
                    span: res.results.length
                });
            }
        });
    }
    /**
     * Free references to a block when it is no longer needed.
     */
    _freeBlock(index) {
        if (!this._dataBlocks[index]) {
            return;
        }
        const offset = index * BLOCK_SIZE;
        const length = this._dataBlocks[index].length;
        delete this._dataBlocks[index];
        this.emitChanged({
            type: 'cells-changed',
            region: 'body',
            row: offset,
            column: 0,
            rowSpan: length,
            columnSpan: this._fieldNames.length
        });
    }
    /**
     * If we are not chunking the data, then just load the whole thing,
     * limited by DEFAULT_LIMIT.
     */
    _fetchDataset() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._connection) {
                return Promise.resolve(void 0);
            }
            const res = yield Private.makeQuery(this._connection, this._query, {
                limit: DEFAULT_LIMIT
            });
            this._fieldNames = res.fields.map((field) => field.name);
            this.emitChanged({ type: 'model-reset' });
            this._tableLength = res.results.length;
            // If the dataset already exists, emit a cells-changed signal.
            // Otherwise, emit a 'rows-inserted' signal.
            if (this._dataset) {
                this._dataset = res.results;
                this.emitChanged({
                    type: 'cells-changed',
                    region: 'body',
                    row: 0,
                    column: 0,
                    rowSpan: res.results.length,
                    columnSpan: this._fieldNames.length
                });
            }
            else {
                this._dataset = res.results;
                this.emitChanged({
                    type: 'rows-inserted',
                    region: 'body',
                    index: 0,
                    span: res.results.length
                });
            }
        });
    }
}
exports.OmniSciTableModel = OmniSciTableModel;
var Private;
(function (Private) {
    /**
     * Create a toolbar. If a connection manager is provided,
     * it will create a change-connection button.
     */
    function createToolbar(widget, editorFactory, manager) {
        const toolbar = new widgets_1.Panel();
        toolbar.addClass('omnisci-OmniSci-toolbar');
        // Create the query editor.
        const queryEditor = new codeeditor_1.CodeEditorWrapper({
            model: new codeeditor_1.CodeEditor.Model(),
            factory: editorFactory
        });
        queryEditor.editor.setOption('lineWrap', 'on');
        queryEditor.editor.model.value.text = '';
        queryEditor.editor.model.mimeType = 'text/x-sql';
        // Create the toolbar.
        toolbar.layout.addWidget(queryEditor);
        toolbar.layout.addWidget(new apputils_1.ToolbarButton({
            className: 'jp-RunIcon jp-Icon jp-Icon-16',
            onClick: () => {
                widget.query = queryEditor.editor.model.value.text;
            },
            tooltip: 'Query'
        }));
        if (manager) {
            toolbar.layout.addWidget(new apputils_1.ToolbarButton({
                className: 'omnisci-OmniSci-logo jp-Icon jp-Icon-16',
                onClick: () => {
                    void manager
                        .chooseConnection('Set SQL Editor Connection', widget.connectionData)
                        .then(connectionData => {
                        return widget.setConnectionData(connectionData);
                    });
                },
                tooltip: 'Enter OmniSci Connection Data'
            }));
        }
        widget.onModelChanged.connect(() => {
            if (widget.query === queryEditor.editor.model.value.text) {
                return;
            }
            queryEditor.editor.model.value.text = widget.query;
        });
        return toolbar;
    }
    Private.createToolbar = createToolbar;
    /**
     * Whether to chunk requests to the backend. We only do this is if
     * (1) LIMIT/OFFSET has not been defined.
     * (2) ORDER BY has been defined, otherwise we cannot guarantee a consistent
     * ordering across requests.
     */
    function shouldChunkRequests(query) {
        return (query.search(/limit/i) === -1 &&
            query.search(/offset/i) === -1 &&
            query.search(/order by/i) !== -1);
    }
    Private.shouldChunkRequests = shouldChunkRequests;
    /**
     * Query the OmniSci backend.
     */
    function makeQuery(connection, query, options = {}) {
        options = Object.assign({ returnTiming: true }, options);
        return new Promise((resolve, reject) => {
            connection.query(query, options, (err, result) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(result);
                }
            });
        });
    }
    Private.makeQuery = makeQuery;
})(Private || (Private = {}));
