"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const signaling_1 = require("@lumino/signaling");
const widgets_1 = require("@lumino/widgets");
const codemirror_1 = require("@jupyterlab/codemirror");
const widget_1 = require("./widget");
const grid_1 = require("./grid");
const vega_lite_1 = require("./vega-lite");
require("../style/index.css");
/**
 * The MIME type for backend-rendered vega.
 */
exports.VEGA_MIME_TYPE = 'application/vnd.omnisci.vega+json';
/**
 * The MIME type for a SQL editor.
 */
exports.SQL_EDITOR_MIME_TYPE = 'application/vnd.omnisci.sqleditor+json';
/**
 * The MIME type for png data.
 */
exports.IMAGE_MIME = 'image/png';
/**
 * A class for rendering a OmniSci-generated image.
 */
class RenderedOmniSciVega extends widgets_1.Widget {
    /**
     * Construct the rendered vega widget.
     */
    constructor() {
        super();
        this.layout = new widgets_1.SingletonLayout();
    }
    /**
     * Render OmniSci image into this widget's node.
     */
    renderModel(model) {
        const layout = this.layout;
        // If we have already rendered a widget, dispose of it.
        if (layout.widget) {
            layout.widget = null;
        }
        // If there is png image data in the mimebundle,
        // we can render that instead of making another
        // request to the backend.
        const imageData = model.data[exports.IMAGE_MIME];
        if (imageData) {
            layout.widget = new widgets_1.Widget({ node: Private.createImageNode(imageData) });
            return Promise.resolve(void 0);
        }
        // Get the data from the mimebundle
        const data = model.data[exports.VEGA_MIME_TYPE];
        const { connection, sessionId, vega, vegaLite } = data;
        // Create a new OmniSciVega
        const vegaWidget = new widget_1.OmniSciVega({
            sessionId,
            connection,
            vega: vega || vega_lite_1.compileToVega(vegaLite),
            vegaLite
        });
        layout.widget = vegaWidget;
        return vegaWidget.renderedImage
            .then(data => {
            // Set the mime data for the png.
            // This allows us to re-use the image if
            // we are loading from disk.
            model.setData({
                data: Object.assign({ 'image/png': data }, model.data),
                metadata: model.metadata
            });
            return void 0;
        })
            .catch(() => void 0); // Catch errors, the default error rendering is OK.
    }
}
exports.RenderedOmniSciVega = RenderedOmniSciVega;
/**
 * A class for rendering a OmniSci-generated image.
 */
class RenderedOmniSciSQLEditor extends widgets_1.Widget {
    /**
     * Construct the rendered sql editor widget.
     */
    constructor() {
        super();
        this.layout = new widgets_1.SingletonLayout();
        this.addClass('omnisci-RenderedOmniSciSQLEditor');
        this._widget = new grid_1.OmniSciSQLEditor({
            editorFactory: Private.editorFactory
        });
        this.layout.widget = this._widget;
    }
    /**
     * Get the underlying SQL editor.
     */
    get widget() {
        return this._widget;
    }
    /**
     * Render the SQL editor into this widget's node.
     */
    renderModel(model) {
        return __awaiter(this, void 0, void 0, function* () {
            // Get the data from the mimebundle
            const data = model.data[exports.SQL_EDITOR_MIME_TYPE];
            if (!data) {
                return Promise.resolve(void 0);
            }
            yield this._widget.grid.setConnectionData(data.connection, data.sessionId);
            this._widget.grid.query = data.query || '';
            return Promise.resolve(void 0);
        });
    }
}
exports.RenderedOmniSciSQLEditor = RenderedOmniSciSQLEditor;
/**
 * A mime renderer factory for omnisci-vega data.
 */
exports.vegaRendererFactory = {
    safe: false,
    mimeTypes: [exports.VEGA_MIME_TYPE],
    defaultRank: 10,
    createRenderer: options => new RenderedOmniSciVega()
};
/**
 * A mime renderer factory for omnisci-sql-editor data.
 */
exports.sqlEditorRendererFactory = {
    safe: false,
    mimeTypes: [exports.SQL_EDITOR_MIME_TYPE],
    defaultRank: 10,
    createRenderer: options => {
        const rendered = new RenderedOmniSciSQLEditor();
        rendered.id = `sql-editor-mime-renderer-${++Private.id}`;
        exports.sqlEditorRendererFactory.rendererCreated.emit(rendered);
        return rendered;
    },
    rendererCreated: new signaling_1.Signal({})
};
const extensions = [
    {
        id: 'jupyterlab-omnisci:vega-factory',
        rendererFactory: exports.vegaRendererFactory,
        dataType: 'string'
    },
    {
        id: 'jupyterlab-omnisci:sqleditor-factory',
        rendererFactory: exports.sqlEditorRendererFactory,
        dataType: 'string'
    }
];
exports.default = extensions;
/**
 * A namespace for private data.
 */
var Private;
(function (Private) {
    /**
     * An incrementing DOM id for sql editor mime renderers.
     * The completer needs a DOM ID for attaching, so we must add one here.
     */
    Private.id = 0;
    /**
     * A default codemirror editor factory for the SQL editor.
     * Since we cannot request the IEditorServices token with a mimeextension,
     * this is a workaround.
     */
    const editorServices = new codemirror_1.CodeMirrorEditorFactory();
    Private.editorFactory = editorServices.newInlineEditor;
    /**
     * Create an image node from a base64-encoded image.
     */
    function createImageNode(blob) {
        const img = document.createElement('img');
        let blobUrl = `data:${exports.IMAGE_MIME};base64,${blob}`;
        img.src = blobUrl;
        return img;
    }
    Private.createImageNode = createImageNode;
})(Private || (Private = {}));
